package player.kent.chen.osgi.launcher;

import java.io.File;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.ServiceLoader;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.osgi.framework.launch.Framework;
import org.osgi.framework.launch.FrameworkFactory;

/**
 * 一个可执行程序，装载OSGi模块并让它们互调
 * 
 * @author chenjianjx
 * 
 */
public class Main {

	public static void main(String[] bundleFiles) throws Exception {

		// 创建launch framework，它是launch osgi程序的入口
		org.osgi.framework.launch.Framework framework = getFramework();

		// 安装service bundle
		Bundle serviceBundle = framework.getBundleContext().installBundle(fileToUri("../play-osgi-service-and-client/service/dist/play-osgi-service.jar.bundle"));

		// 安装client bundle
		Bundle clientBundle = framework.getBundleContext().installBundle(fileToUri("../play-osgi-service-and-client/client/dist/play-osgi-client.jar.bundle"));

		// 启动framework和bundles
		framework.start();
		serviceBundle.start();
		clientBundle.start();

		// 启动client
		Class<?> clientClass = clientBundle.loadClass("player.kent.chen.osgi.client.HelloClient");
		Method method = clientClass.getMethod("callHelloService", new Class<?>[]{});
		method.invoke(clientClass.newInstance());

		// 最后停止framework
		framework.stop();
	}

	private static String fileToUri(String file) {

		return new File(file).toURI().toString();

	}

	@SuppressWarnings("rawtypes")
	private static Framework getFramework() throws BundleException {
		FrameworkFactory frameworkFactory = getFrameworkFactory();

		final Framework framework = frameworkFactory.newFramework(new HashMap());

		framework.init();

		return framework;
	}

	private static FrameworkFactory getFrameworkFactory() {
		Class<FrameworkFactory> theClass = FrameworkFactory.class;
		ServiceLoader<FrameworkFactory> factoryLoader = ServiceLoader.load(theClass);

		FrameworkFactory frameworkFactory = factoryLoader.iterator().next();
		return frameworkFactory;
	}

}